#include    <system.h>
#include    "50.h"

/*FINAL FUNCTION LIBRARY
The 50, Senior Design 2006-07

Software Development Team:
Michael J. Gerardi
Martin Nguyen*/

void LCD_init(void)
{
  trisd = trisd & 0x0f;  //Top 4 pins configured for output
  trisc.2 = 0;			 //trisc.2 goes to 
  trisc.5 = 0;
  //trisc = trisc & 0xDB; // 1101 1011 
  
  delay_ms(40);

  LCD_icmd( 0x30 ) ;          /*  8 Bit mode  */
  delay_ms(5);			// DH - min delay here of 4.1 ms
  LCD_icmd( 0x30 ) ;          /*  8 Bit mode  */
  LCD_icmd( 0x30 ) ;          /*  8 Bit mode  */

  LCD_icmd( 0x20 ) ;          /*  4-BIT Mode  */
  LCD_cmd( 0x28 );            /*  Function Set: 4-bit,2-line,5X7 */

  LCD_cmd( 0x0C );            /*  Display on, Cursor off         */
  LCD_cmd( 0x06 );            /*  Entry mode: INC addr, NO SHIFT display */

  LCD_cmd( 0x01 );            /*  Clear Display */
  delay_ms( 20 );
}

/****************************************************
Initializes the I2C serial protocol, which connects
the memory and time chip to the microcontroller
*****************************************************/

void i2c_init(void){

sspstat.7 = 1;	/*Sample @ end of data output time*/
sspstat.6 = 0;  /*Transition occurs on move from idle to active*/
sspcon1.5 = 1;  /*SPEN */

/*These 4 bits set the mc to i2c master mode*/
sspcon1.3 = 1;  
sspcon1.2 = 0;
sspcon1.1 = 0;
sspcon1.0 = 0;

sspadd = 0x63; // For 100 kHz serial transmission

}

/*******************************************************
*  term_init - Initializes the drivers on the microcont-*
*  roller to prepare for serial transmission of data.  
*  (This was written for our taskwork, but serial transmission
*  will become important in the future if we want to do a 
*  serial dump of the data in memory to provide feedback on
*  car performance.)
*******************************************************/
void term_init(void)
{
	txsta |= 0x24; 		//00100100
	rcsta |= 0x90;		//10010000
	baudcon |= 0x08; 	//00001000
	spbrgh = 0x00;		//00000000
	spbrg = 42;			//00101010

}

/**********************************************;
This function enables global and peripheral interrupts.
It also sets the mode for sleep mode.  Initially, the only
interrupt (B1) is DISABLED to prevent a transient interrupt
from disturbing system performance at an inappropriate time.
The interrupt is turned on in the interrupt_init_sleep function,
right before 
**********************************************/

void int_wdt_init(void)
{
	intcon.7 = 1;  //global interrupt enable
	intcon.6 = 1;  //peripheral interrupt enable
	
	//To prevent an interrupt from occuring at a point in the program where they are unwarranted,
	//we initially disable the interrupt.
	intcon3.3 = 1; //INT1 external interrupt ENABLED.
	
	osccon = 0xF0;	//will go to idle when sleep command is executed
					//PR_IDLE--continues to clock peripherals after sleep is executed
	wdtcon.0 = 0;	//Disable watchdog timer;

}

/*This function activates and appropriately orients the alternator and override 
interrupts
(high-to-low, low-to-high) before the program enters idle mode.*/	

void int_wdt_init_sleep(void)
{
	volatile bit user_int@INTCON3.0;
	
	osccon = 0xF0;	//will go to idle when sleep command is executed
					//PR_IDLE--continues to clock peripherals after sleep is executed
	
	intcon.7 = 1;  //global interrupt enable
	intcon.6 = 1;  //peripheral interrupt enable
	
	intcon3.3 = 1;
	user_int = 0;
	wdtcon.0 = 1;	
}	

/*******************************************************
*  initialize_ad--sets register bits relevant to performing*
*  AD conversion on the 4620.*
*******************************************************/

void ad_init(void){

// Configure the A/D module:

adcon1 = 0x06;
// -Configure analog pins, voltage reference and digital I/O (ADCON1)
//  00|00|0110
//  This accounts for nine possible A/D inputs


// - Select A/D input channel (ADCON0) 
//   do this specifically elsewhere

adcon2 = 0x0F;
// - Select A/D acquisition time (ADCON2)
// - Select A/D conversion clock (ADCON2)
// 00|001|111

adcon0.0 = 1;
adcon0.1 = 0; // - Turn on A/D module (ADCON0)
}

/**********************************************************************
Initializes settings to ready all digital in/out ports to act as outputs,
for the purpose of turning on / switching off devices.
***********************************************************************/

void io_init(void){

volatile bit pin1@LATD.0;
volatile bit pin2@LATD.1;
volatile bit pin3@LATD.2;
volatile bit pin4@LATD.3;
volatile bit pin5@LATC.0;
volatile bit pin6@LATC.1;

unsigned short switchoff = 0;	//switch off device
unsigned short switchon = 1;		//switch on device

/*These settings define output ports that will enable or disable various
various devices*/

trisd.0 = 0;
trisd.1 = 0;
trisd.2 = 0;
trisd.3 = 0;
trisc.0 = 0;
trisc.1 = 0;

pin1 = switchoff;
pin2 = switchoff;
pin3 = switchoff;
pin4 = switchoff;
pin5 = switchoff;
pin6 = switchoff;
}

/**********************************************************************/
/*LCD COMMANDS (Mostly provided @ beginning of year*/
/**********************************************************************/

/*******************************************************
 *  LCD_cmd - Write ASCII character to LCD peripheral  *
 *      configured as a 4 bit interface                *
 *******************************************************/
void LCD_char(char data)
{
  char dsave;
  dsave = latd;
  LCD_DATA = (data & 0xF0) | (LCD_DATA & 0x0F);         /* Write HI nibble word to LCD */
  write_data;
  LCD_DATA = ((data << 4) & 0xF0) | (LCD_DATA & 0x0F);  /* Write LO nibble word to LCD */
  write_data;
  delay_ms(1);
  latd = dsave;
}
/*****************************************************
 *  LCD_icmd - Write control word to LCD peripheral  *
 *****************************************************/
void LCD_icmd(char data)
{
  char dsave;
  dsave = portd;
  LCD_DATA = data | (LCD_DATA & 0x0F) ;
  write_cmd;

  delay_ms(1);
  portd = dsave;
  return;
}

/***************************************************
 *  LCD_cmd - Write control word to LCD peripheral *
 *      configured as a 4 bit interface            *
 **************************************************/
void LCD_cmd(char data)
{
  char dsave;
  dsave = portd;
  LCD_DATA = (data & 0xF0) | (LCD_DATA & 0x0F);         /* Write HI nibble word to LCD */
  write_cmd;
  LCD_DATA = ((data << 4) & 0xF0) | (LCD_DATA & 0x0F);  /* Write LO nibble word to LCD */
  write_cmd;
  if (data < 0x03) delay_ms(20); else delay_ms(1);
  portd = dsave;
  return;

}
/* set the cursor position */

void LCD_setpos(char line, char pos)
{
  LCD_cmd(0x80 + line * 0x40 + pos);
  return;
}

/**********************
 * routine to clear display screen
 * */
 
void LCD_clear(void)
{
  LCD_cmd( 0x01 );            /*  Clear Display */
  delay_ms(20);
  return;
}

/********************************
 * Display char in HEX - format *
 ********************************/
void LCD_hex(char data)
{
  char n;
  n = ((data >> 4) & 0x0F) + 0x30;
  if (n > 0x39) n = n+7;
  LCD_char(n);
  n = (data & 0x0F) + 0x30;
  if (n > 0x39) n = n+7;
  LCD_char(n);
  return;
}
/********************************
 * Display unsigned short in HEX - format *
 ********************************/
void LCD_hex(unsigned short	  data)
{
  char n;
  n = ((data >> 12) & 0x0F) + 0x30;
  if (n > 0x39) n = n+7;
  LCD_char(n);
  n = ((data >> 8) & 0x0F) + 0x30;
  if (n > 0x39) n = n+7;
  LCD_char(n);
  n = ((data >> 4) & 0x0F) + 0x30;
  if (n > 0x39) n = n+7;
  LCD_char(n);
  n = (data & 0x0F) + 0x30;
  if (n > 0x39) n = n+7;
  LCD_char(n);
  return;
}
/********************************
 * Display  short in HEX - format *
 ********************************/
void LCD_hex(short data)
{
  char n;
  n = ((data >> 12) & 0x0F) + 0x30;
  if (n > 0x39) n = n+7;
  LCD_char(n);
  n = ((data >> 8) & 0x0F) + 0x30;
  if (n > 0x39) n = n+7;
  LCD_char(n);
  n = ((data >> 4) & 0x0F) + 0x30;
  if (n > 0x39) n = n+7;
  LCD_char(n);
  n = (data & 0x0F) + 0x30;
  if (n > 0x39) n = n+7;
  LCD_char(n);
  return;
}
/********************************
 * Display low nibble in HEX - format *
 ********************************/
void LCD_nib(char data)
{
  char n;
  n = (data & 0x0F) + 0x30;
  if (n > 0x39) n = n+7;
  LCD_char(n);
  return;
}

/********************************
 * Display byte in binary - format *
 ********************************/
void LCD_bin(char data)
{
  char n;
  char temp;
  temp =0x80;
  for( n=1; n<=8; ++n)
  {
    if(temp & data)
      LCD_char(0x31);
    else
      LCD_char(0x30);
    temp = temp >> 1;
  }
  return;
}
/********************************
 * Display unsigned short in binary - format *
 ********************************/
void LCD_bin(unsigned short data)
{
  char n;
  unsigned short temp;
  temp =0x8000;
  for( n=1; n<=16; ++n)
  {
    if(temp & data)
      LCD_char(0x31);
    else
      LCD_char(0x30);
    temp = temp >> 1;
  }
}

void LCD_dec(short dat)
{
  unsigned short val;   // ascii results
  unsigned short temp;
  unsigned short div;
  unsigned short data;
  char i;
  char digit;
  data = dat;  // make it unsigned
  div = 10000;
  for(i=0; i <= 4; ++i)   // get all 5 digits
  {
    val = data/div;       // get most signif. digit
    LCD_char(val + '0');    // print digit
    data -= val * div;   // what weve printed
    div=div/10;         // adjust divisor
  }
  return;
}

void LCD_dec(unsigned short dat)
{
  unsigned short val;   // ascii results
  unsigned short temp;
  unsigned short div;
  unsigned short data;
  char i;
  char digit;
  data = dat;  // make it unsigned
  div = 10000;
  for(i=0; i <= 4; ++i)   // get all 5 digits
  {
    val = data/div;       // get most signif. digit
    LCD_char(val + '0');    // print digit
    data -= val * div;   // what weve printed
    div=div/10;         // adjust divisor
  }
  return;
}
/**********************************
 * Display byte as decimal number *
 **********************************/
void LCD_dec(char data)
{
  LCD_char(((data /100) & 255) + 0x30);
  data = data % 100;
  LCD_char( ((data / 10) & 255) + 0x30 );
  LCD_char( ((data % 10) & 255) + 0x30 );
  return;
}
/*
 * display char as a signed int
 * */

void LCD_int(char data)
{
  char val;   // ascii results
  char temp;
  char div;
  char i;
  bool dozero;
  dozero = false;
  div = 100;
  if(data == 0)  // always print 0
  {
    LCD_char('0');
    return;
  }
    /*
     * adjust for sign
     * */
  if(data & 10000000b)
  {
    LCD_char('-');
    data = ~data + 1;
  }
  for(i=0; i <= 2; ++i)   // get all 3 digits
  {
    val = data/div;       // get most signif. digit
    if(val != 0 || dozero)
    { 
      LCD_char(val + '0');    // print digit
      dozero = true;
    }
    temp = val * div;   // subtract off
    data = data - temp;   // what weve printed
    div=div/10;         // adjust divisor
  }
}
/*
 * display unsigned short as a signed int
 * */

void LCD_int(short dat)
{
  short val;   // ascii results
  short temp;
  short div;
  char i;
  unsigned short data;
  bool dozero;
  dozero = false;
  data = dat;
  if(dat == 0)  // always print 0
  {
    LCD_char('0');
    return;
  }
  div = 10000;
    /*
     * adjust for sign
     * */
  if(dat < 0)
  {
    LCD_char('-');
    data = ~data + 1;
  }
  for(i=0; i <= 4; ++i)   // get all 3 digits
  {
    val = data/div;       // get most signif. digit
    if(val != 0 || dozero) 
    {
      LCD_char(val + '0');    // print digit
      dozero = true;
    }
    data -= val*div;   // subtract off what weve printed
    div=div/10;         // adjust divisor
  }
}


void LCD_int(unsigned short dat)
{
  short val;   // ascii results
  short temp;
  short div;
  char i;
  unsigned short data;
  bool dozero;
  dozero = false;
  data = dat;
  if(dat == 0)  // always print 0
  {
    LCD_char('0');
    return;
  }
  div = 10000;
    /*
     * adjust for sign
     * */
  for(i=0; i <= 4; ++i)   // get all 3 digits
  {
    val = data/div;       // get most signif. digit
    if(val != 0 || dozero) 
    {
      LCD_char(val + '0');    // print digit
      dozero = true;
    }
    data -= val*div;   // subtract off what weve printed
    div=div/10;         // adjust divisor
  }
}

void LCD_printf( const char* text )
{
  char i = 0;
  while( text[i] != 0 )
    LCD_char( text[i++] );
  return;
}

/************************************************************/
/*SERIAL PORT INTERFACE (Mostly provided @ beginning of year*/
/************************************************************/

/*******************************************************
putc - puts a character in the microcontroller on the
the terminal
*******************************************************/

void putc (char value)
{
	volatile bit thing@PIR1.4;
	while (!thing);
	txreg = value;
}

/*******************************************************
getc - gets a character from terminal input and places
it in the serial transmission register.
*******************************************************/

char getc(void)
{
	volatile bit thing@PIR1.5;
	short temp;
	while (!thing) {};
		temp = rcreg;
		putc(temp);
		return temp;
}

/*******************************************************
term_printf--prints a string to the terminal using term_
char.
*******************************************************/


void term_printf( const char* text )
{
  char i = 0;
  while( text[i] != 0 ){
	putc(text[i++]);
	}
}

/*******************************************************
term_dec prints the value of an unsigned short variable
to the terminal.
*******************************************************/

void term_dec(unsigned short dat)
{
  unsigned short val;   // ascii results
  unsigned short temp;
  unsigned short div;
  unsigned short data;
  unsigned short prev = 0;
  char i;
  char digit;
  bool lead = false;
  data = dat;  // make it unsigned
  div = 10000;
  
  for(i=0; i <= 4; ++i)   // get all 5 digits
  {
    val = data/div;	
    if(lead == false){		// get most signif. digit
		if(val != 0){
			lead = true;
			}
		}
	if(lead == true){
		putc(val + '0');    // print digit
		  
		}
	data -= val * div; 	// what weve printed
	div=div/10; 		// adjust divisor
  }
  if(lead == false){
	term_printf("0");
	}
  return;
}

/*******************************************************
getnum--prints the value of an unsigned short val-
ue.
*******************************************************/

unsigned short getnum(void)
{
	char val;
	char temp;
	char j;
	val = 0;
	j = 0;
	do
	{
		temp = getc(); // get value
		//putc(temp); //echo that, dude
		if(temp == '\r') return(val);
		val *= 10;
		val += (temp - '0');
		} while(1);
}

/*term_menu.  This is a basic user interface that works in Terminal.  It provides a link between the user and the memory
dump routine of the microcontroller.  It passes the needed memory locations to the dump function in order to provide
reference points for the dump*/

void term_menu(bool read_all, unsigned short ad_high, unsigned short ad_low, unsigned short ad_high_end, unsigned short ad_low_end){

	bool loop = true;
	bool q_loop = true;
	bool c_loop = true;
	bool p_loop = true;
	unsigned short cont = 0;
	unsigned short numeral = 0;
	unsigned short quit = 0;
	unsigned short term_read = 0;
	
	putc(0x0C);
	term_printf("Police Car Rx\n\r");
	term_printf("Copyright, 5-0 Productions, 2007\n\r");
	term_printf("  \n\r");
	while(loop){
		term_printf("User Options:\n\r");
		putc(0x09);
		term_printf("(1) Dump Measurements\n\r");
		putc(0x09);
		term_printf("(2) Exit\n\r");
		term_printf("Please make a selection and press 'enter': ");
		numeral = getnum();
		if (numeral == 1){
			while(p_loop){
				term_printf("Dump Options:\n\r");
				putc(0x09);
				term_printf("(1) Begin Dump of Recent Measurements\n\r");
				putc(0x09);
				term_printf("(2) Dump Entire Memory (if power has failed)\n\r");
				term_printf("Please make a selection and press 'enter': ");
				numeral = getnum();
				if (numeral == 1){
					p_loop = false;
					}
				if (numeral == 2){
					read_all = true;
					p_loop = false;
				}
				if ((numeral != 1) && (numeral != 2)){
					putc(0x0C);
					term_printf("Invalid Entry.  Please Select From The Following Options:\n\r");
					}
			}	
			putc(0x0C);
			dump_mem(read_all, ad_high, ad_low, ad_high_end, ad_low_end);
			term_read = 1;
			putc(0x0C);
			term_printf("Task Complete. Quit or Continue?\n\r");
			while(c_loop == true){
				putc(0x09);
				term_printf("(1) Quit\n\r");
				putc(0x09);
				term_printf("(2) Continue\n\r");
				term_printf("Please make a selection and press 'enter': ");
				cont = getnum();	
				if(cont == 1){
					loop = false;
					c_loop = false;
					}
				if(cont == 2){
					c_loop = false;
					}
				if(cont != 1 & cont != 2){
					putc(0x0C);
					term_printf("Invalid Entry.  Please Select From The Following Options:\n\r");
					}
			}
			cont = 0;
			c_loop = true;
			putc(0x0C);
			}						
		if (numeral == 2){
			putc(0x0C);
			term_printf("Are you sure you would like to quit?\n\r");
			while(q_loop == true){
				putc(0x09);
				term_printf("(1) Yes\n\r");
				putc(0x09);
				term_printf("(2) No\n\r");
				term_printf("Please make a selection and press 'enter': ");
				quit = getnum();
				if(quit == 1){
					loop = false;
					q_loop = false;
					}
				if(quit == 2){
					q_loop = false;
					}
				if(quit != 1 & quit != 2){
					putc(0x0C);
					term_printf("Invalid Entry.  Please Select From The Following Options:\n\r");
					}
				}
			putc(0x0C);
			quit = 0;
			q_loop = true;
			}
		if (numeral != 1 & numeral != 2){
			putc(0x0C);
			term_printf("Invalid Entry.  Please Select From The Following Options:\n\r");
		}
		numeral = 0;
	}
	putc(0x0C);
	term_printf("Goodbye!");
	// return term_read;
	write_term_read(term_read);
}

/*dump_mem.  This program reads the EEPROM memory in blocks, and dumps it into comma separated format in HyperTerminal.
With the correct HyperTerminal settings, this dump will result in a comma-separated data file that can be dumped into
Microsoft Excel or OpenOffice, allowing for simplified analysis by users in a highly standardized format*/

void dump_mem(bool read_all, unsigned short ad_high, unsigned short ad_low, unsigned short ad_high_end, unsigned short ad_low_end){
	
	volatile bit thing@PIR1.3;
	short stat = 1;
	short size = 22;
	bool bust = false;
	unsigned short read_dec = 0;
	
	unsigned short ad_high_start;
	unsigned short ad_low_start;
	
	ad_high_start = ad_high;
	ad_low_start = ad_low;
	
	unsigned short minute;
	unsigned short hour;
	unsigned short date;
	unsigned short month;
	
	unsigned short L1_h = 0;
	unsigned short L1_l = 0;
	unsigned short L2_h = 0;
	unsigned short L2_l = 0;
	unsigned short L3_h = 0;
	unsigned short L3_l = 0;
	unsigned short R1_h = 0;
	unsigned short R1_l = 0;
	unsigned short R2_h = 0;
	unsigned short R2_l = 0;
	unsigned short R3_h = 0;
	unsigned short R3_l = 0;
	unsigned short alt = 0;
	unsigned short bat_a = 0;
	unsigned short bat_v_h = 0;
	unsigned short bat_v_l = 0;
	
	unsigned short L1;
	unsigned short L2;
	unsigned short L3;
	unsigned short R1;
	unsigned short R2;
	unsigned short R3;
	unsigned short bat_v;
	unsigned short soc;
	
	unsigned short status;
	unsigned short fault;
	
	if(read_all == true){
		read_dec = 1;
		}
	else{
		read_dec = 2;
		}
	
	term_printf("Month/Date, Time of Measurement, L1, L2, L3, R1, R2, R3, Alt(A), Bat(A), Bat(V), SOC, Car, Override, Faults \n\r");
	
/*Prelude--indicate starting address for read, initial read/write commands*/
	while(bust == false){		//Wait for slave write to finish
	
		while(stat == 1){		//Wait for slave to finish what it's doing
			i2c_start();
			sspbuf = 0xA0;		//Write to memory address command
			while (thing == 0) {}	
			if (sspcon2.6 == 1){	//If NOT ACK (still writing)
				thing = 0;
				sspcon2.2 = 1;		//Send STOP
				while(thing == 0){}
					thing = 0;
				}
			else{					//If ACK (sspcon2.6 = 0)
				stat = 0;			//Break while loop
				}
			}
		stat = true;
		thing = 0;
		sspbuf = ad_high;		
		while(thing == 0){}
		thing = 0;	
		sspbuf = ad_low;
		while(thing == 0){}
		thing = 0;
		sspcon2.2 = 1;
		while(thing == 0){}
		thing = 0;
		
		
		/*This command initiates a read*/
		sspcon2.0 = 1;	
		while(thing == 0){}
		thing = 0;
		sspbuf = 0xA1;
		while (thing == 0) {}
		thing = 0;
		
		minute = i2c_send_ack();	//Variables associated w/ time stamp
		hour = i2c_send_ack();
		date = i2c_send_ack();
		month = i2c_send_ack();
		L1_h = i2c_send_ack();		//Current L1
		L1_l = i2c_send_ack();
		L2_h = i2c_send_ack();		//Current L2
		L2_l = i2c_send_ack();
		L3_h = i2c_send_ack();		//Current L3
		L3_l = i2c_send_ack();
		R1_h = i2c_send_ack();		//Current R1
		R1_l = i2c_send_ack();
		R2_h = i2c_send_ack();		//Current R2
		R2_l = i2c_send_ack();
		R3_h = i2c_send_ack();		//Current R3
		R3_l = i2c_send_ack();
		alt = i2c_send_ack();		//Alternator Current
		bat_a = i2c_send_ack();		//Battery Current
		bat_v_h = i2c_send_ack();	//Battery Voltage
		bat_v_l = i2c_send_ack();
		status = i2c_send_ack();	//System Status
	
		/*Fault Indicator*/
		sspcon2.3 = 1;
		while (thing == 0)	{}
		thing = 0;
		fault = sspbuf;
		sspcon2.4 = 1;		
		sspcon2.5 = 1;			//for final word transmitted, send NOT ACK
		while (thing == 0)	{}
		thing = 0;	
		sspcon2.2 = 1;
		while( thing == 0) {}
		thing = 0;

/******************Memory Pointer Adjustment/Continuation Logic**************************/
		/*Increment memory placeholders for next pass through loop*/
		
		if ((ad_low + size) > 255){
			ad_low = 0x00;
			ad_high = ad_high + 1;
			if (ad_high == 256){
				ad_high = 0;
				}
			}
		else{
			ad_low = ad_low + size;
			}
			
		/*If we haven't filled the memory, break out when we've read to the last entry*/
		if(read_dec == 2){
			if(ad_high == ad_high_end){
				if(ad_low == (ad_low_end)){
					bust = true;
					}
				}
			}
		/*If we have filled the memory, read all the way around.*/
		if(read_dec == 1){
			if(ad_high == ad_high_start){
				if(ad_low == (ad_low_start)){
					bust = true;
					}
				}
			}
/***************************Reconstitute Currents/Voltage**********************/
	
	L1 = (L1_h * 256) + (L1_l);		//compress currents into single short variable
	L2 = (L2_h * 256) + (L2_l);
	L3 = (L3_h * 256) + (L3_l);
	R1 = (R1_h * 256) + (R1_l);
	R2 = (R2_h * 256) + (R2_l);
	R3 = (R3_h * 256) + (R3_l);
	bat_v = (bat_v_h * 256) + bat_v_l;
	
	soc = ((bat_v - 11850) / 8);
	if(soc > 100){
		soc = 100;
	}
	
	/*Printing time*/   
		term_dec(month);
		term_printf("/");
		term_dec(date);
		term_printf(", ");
		if (hour < 10){
			term_printf(" 0");
			term_dec(hour);
			}
		else{
			term_printf(" ");
			term_dec(hour);
			}
		term_printf(":");
		if (minute < 10){
			term_printf("0");
			term_dec(minute);
			}
		else{
			term_dec(minute);
			}
		term_printf(", ");
/*Printing Readings*/
	if((status & 0x0080) == 1){	//L1
		term_printf("On ");
		}
	else{
		term_printf("Off ");
		}
		print_ad_term_amp(L1, 1);
		term_printf(", ");
		
	if((status & 0x0040) == 1){	//L2
		term_printf("On ");
		}
	else{
		term_printf("Off ");
		}
		print_ad_term_amp(L2, 1);
		term_printf(", ");
		
	if((status & 0x0020) == 1){	//L3
		term_printf("On ");
		}
	else{
		term_printf("Off ");
		}
		print_ad_term_amp(L3, 1);
		term_printf(", ");
		
	if((status & 0x0010) == 1){	//R1
		term_printf("On ");
		}
	else{
		term_printf("Off ");
		}
		print_ad_term_amp(R1, 1);
		term_printf(", ");
		
	if((status & 0x0008) == 1){	//R2
		term_printf("On ");
		}
	else{
		term_printf("Off ");
		}
		print_ad_term_amp(R2, 1);
		term_printf(", ");
		
	if((status & 0x0004) == 1){	//R3
		term_printf("On ");
		}
	else{
		term_printf("Off ");
		}
		print_ad_term_amp(R3, 1);
		term_printf(", ");
		
		print_ad_term_amp(alt, 2);	//alternator current
		term_printf(", ");
		
		print_ad_term_amp(bat_a, 2);	//battery current
		term_printf(", ");
		
		print_ad_term_volt(bat_v);	//battery voltage
		term_printf(", ");
		
		term_dec(soc);				//battery state of charge
		term_printf(", ");
		
/*Print Car On/Off, Override status*/
	if((status & 0x0002) == 1){	//Car on/off
		term_printf("On, ");
		}
	else{
		term_printf("Off, ");
		}
	if((status & 0x0001) == 1){	//System override
		term_printf("On, ");
		}
	else{
		term_printf("Off, ");
		}
/*Print fault status*/
	if(fault == 3){
		term_printf("Alt/Bat Failure + Low SOC");
		}
	if(fault == 2){
		term_printf("Alt/Bat Failure");
		}
	if(fault == 1){
		term_printf("Low SOC");
		}
	if(fault == 0){
		term_printf("Normal");
		}
		term_printf("\n\r");
	}
}

/*AD FUNCTIONS********************************************************/

/*******************************************************
*  check_batt_voltage--runs the ad converter on 4620*
*  to check the voltage on the battery (duh).*
*******************************************************/

unsigned short check_ad(short pin_select){

//pin 2

unsigned short voltage;
unsigned short final;
unsigned long output;
short temp;
bool ign = false;
char val;

volatile bit go_done@ADCON0.1;

//initialize_ad();
// 00|0000|01;
adcon0 = pin_select;
delay_ms(100);
go_done = 1;
while(go_done){}
voltage = adresh*256 + adresl;
adcon0.0 = 0;

voltage = volt_scaler(voltage);

/*Large Current Sensors on alternator and battery.  Ranges from 0 to 793 A)*/
if (pin_select == 0x19 | pin_select == 0x1D){
	temp = voltage;
	temp = (temp - 2500);	//2.5 V offset
	if(temp <= 0){
		temp = 0;
	}
	output = temp;
	output = output * 200;		//base current of sensor
	output = output / 625;		//reflects scaling factor of .625
	final = (unsigned int)(output & 0x0000FFFF);	//convert to short variable.
	ign = true;
	}
/*Voltage reader off of op-amp.*/

if (pin_select == 0x21){
	final = voltage * 3;		//Measured voltage will be scaled down by 3
	ign = true; 
	}

/*All other pins connected to Allegro sensors.*/
if (ign == false){	
	temp = voltage;
	temp = temp - 600;	//.6 V quiescent voltage
	if (temp <= 0){
		temp = 0;
		}
	voltage = temp;
	final = voltage / 60;
	voltage = voltage % 60;
	temp = (voltage / 6);
	final = (final * 10) + temp;      		//.06 V per amp.
	}
	
return final;
}

/*********************************************
 Scales unsigned short raw data to range of 0-5 Volts
 *********************************************/
unsigned short volt_scaler(unsigned short raw){

	unsigned short volts;
	
	unsigned short vones;
	unsigned short vtenths;
	unsigned short vhunds;
	unsigned short vthous;
	
	vones = raw/13094; //13094 = 65470 / 5 (approx)
	raw = raw % 13094;
	vtenths = raw/1309;  //1309 = 13094 / 10 (approx)
	raw = raw % 1309;
	vhunds = raw/131;	//131 = 1390 / 10 (approx)
	raw = raw % 131;
	vthous = raw / 13;  //13 = 131 / 10 (approx)
	
	volts = (1000 * vones) + (100 * vtenths) + (10 * vhunds); 	//combine to form number from 000-500
	
	return volts;

}

/************************************************
Prints the voltage measured from the AD converter to the LCD screen,
after downconverting it into ones, tenths, hundredths
*************************************************/
void print_ad_LCD(unsigned short pin_select){

unsigned short final = 0;
bool ign = true;
final = check_ad(pin_select);

if(pin_select == 0x21){
	unsigned short vtens;
	unsigned short vones;
	unsigned short vtenths;
	unsigned short vhunds;
	
	vtens = final / 10000;
	LCD_int(vtens);
	final = final % 10000;
	vones = final / 1000;	//Extracts ones place from input 
	LCD_int(vones);
	LCD_printf(".");		//Decimal place gives impression of FP number
	final = final % 1000;
	vtenths = final / 100;	//Extract tenths place
	LCD_int(vtenths);
	final = final % 100;	//Extract hundredths place
	vhunds = final / 10;
	LCD_int(vhunds);
	LCD_printf(" V");
	ign = false;
	}
		
if((pin_select != 0x21) && (pin_select != 0x1D) && (pin_select != 0x19) && (pin_select != 0x19) && (pin_select != 0x1D)){	
//if((pin_select != 0x21) || (pin_select != 0x1D) || (pin_select != 0x19)){
	unsigned short aint;
	unsigned short afrac;
	
	aint = final / 10;
	final = final % 10;
	LCD_int(aint);
	LCD_printf(".");
	afrac = final;
	LCD_int(afrac);
	LCD_printf(" A");
	ign = false;
	}
	
if(ign == true){
	LCD_int(final);
	LCD_printf(" A");
	}

}

/************************************************
Prints the voltage measured from the AD converter to the terminal,
after downconverting it into ones, tenths, hundredths
*************************************************/
void print_ad_term_volt(unsigned short dingus){
	
//dingus is a value ranged 000 - 15000, interpreted as 0.00 - 15.00 V

	unsigned short vtens;
	unsigned short vones;
	unsigned short vtenths;
	unsigned short vhunds;
	
	vtens = dingus / 10000;
	term_dec(vtens);
	dingus = dingus % 10000;
	vones = dingus / 1000;	//Extracts ones place from input 
	term_dec(vones);
	term_printf(".");		//Decimal place gives impression of FP number
	dingus = dingus % 1000;
	vtenths = dingus / 100;	//Extract tenths place
	term_dec(vtenths);
	dingus = dingus % 100;	//Extract hundredths place
	vhunds = dingus / 10;
	term_dec(vhunds);
}

void print_ad_term_amp(unsigned short value, unsigned short type){
	unsigned short aint;
	unsigned short afrac;
	
	if(type == 1){ 		//small, Allegro current sensors
	aint = value / 10;
	value = value % 10;
	term_dec(aint);
	term_printf(".");
	afrac = value;
	term_dec(afrac);
	term_printf(" A");
	}
	
	if(type == 2){		//large, LEM current sensors
	term_dec(value);
	term_printf(" A");
	}		
}

unsigned short soc_check(void){
	unsigned short raw;
	unsigned short soc;
	raw = check_ad(0x21);
	soc = ((raw - 11850) / 8);
	if(soc > 100){
		soc = 100;
	}
	return soc;
}

/*I2C FUNCTIONS*******************************************************/

/*this set of functions is meant to replace some basic operations in doing I2C
 transmission.  After integrating them, I had to take them out in an attempt to 
debug something and haven't reintegrated them (there was nothing wrong with the 
functions themselves), but that will be accomplished eventually*/

void i2c_start(void){
	volatile bit thing@PIR1.3;  //set to one whenever an event happens

	sspcon2.0 = 1;              //initializes start mode
	while ( thing == 0) {}      //waits for flag, then resets.
	thing = 0;
}

void i2c_stop(void){
	volatile bit thing@PIR1.3;
	sspcon2.2 = 1;             //initializes stop mode
	while( thing == 0) {}
	thing = 0;
}

/*these two functions are used in serial reads, to acknowledge a received bit--it might hae to be modified to allow a variable to be passed through it, because the value read from the time chip or memory might need to be passed to a variable before the acknowledge sequence is begun.*/

unsigned short i2c_send_ack(void){
	volatile bit thing@PIR1.3;
	unsigned short value;
	
	sspcon2.3 = 1;
	while (thing == 0){}
	thing = 0;
	sspcon2.4 = 1;		
	sspcon2.5 = 0;	
	value = sspbuf;
	while (thing == 0)	{}
	thing = 0;
	
	return value;
}

void i2c_send_not_ack(void){
	volatile bit thing@PIR1.3;
	sspcon2.4 = 1;              //send an interrupt
	sspcon2.5 = 1;              //send not acknowledge
	while (thing == 0)	{}
	thing = 0;
}

/*This needs to be set before any value is received off the I2C line*/

void i2c_receive_mode(void){
	volatile bit thing@PIR1.3;
	sspcon2.3 = 1;               //receive mode enable
	while (thing == 0)	{}
	thing = 0;
}

/*Sets the time registers we're concerned with.  At some point in the future, this will have to be done with some sort of user input, like a touchpad, but for now, this demonstrates how a write works, for the sake of demonstration*/

void i2c_time_write(short addr, short data){

	volatile bit thing@PIR1.3;
	i2c_start();
	sspbuf = 0xD0; /* 11010000, code for time chip WRITE*/
	while( thing == 0) {}
	 thing = 0;
	sspbuf = addr;
	while( thing == 0) {}
	thing = 0;
	sspbuf = data;
	while( thing == 0) {}
	thing = 0;
	i2c_stop();
}

/*Old test function to read seconds off time chip; now redundant*/

short i2c_time_read_seconds(void){

	short seconds;
	volatile bit thing@PIR1.3;
	
	i2c_start();
	sspbuf = 0xD0;		/*slave address WRITE*/
	while( thing == 0)   {}
	thing = 0;
	sspbuf = 0x00;		/*seconds register*/
	while ( thing == 0)  {}
	thing = 0;	
	sspcon2.1 = 1;		/*restart condition*/
	while ( thing == 0)  {}
	thing = 0;	
	sspbuf = 0xD1;		/*slave address READ*/
	while ( thing == 0)  {}
	thing = 0;
	i2c_receive_mode();
	i2c_send_not_ack();
	i2c_stop();
	seconds = sspbuf;
	return seconds;
}

/*basic digital clock function; reads seconds, minutes, hours, date, and month;
 processes results into numerically sensical values, and prints them on the LCD.*/

unsigned short i2c_time(bool write, unsigned short address_low, unsigned short address_high){

/*PARAMETERS*/
	unsigned short minutes;
	unsigned short hours;
	unsigned short date;
	unsigned short month;
	unsigned short seconds;
	unsigned short day;
	unsigned short hours_temp;
	
	volatile bit thing@PIR1.3;

/*RETREIVAL SEQUENCE*/

	sspcon2.0 = 1;
	while ( thing == 0) {}
	thing = 0;
	sspbuf = 0xD0;		/*slave address WRITE*/
	while( thing == 0)   {}
	thing = 0;
	sspbuf = 0x00;		/*seconds register*/
	while ( thing == 0)  {}
	thing = 0;	
	sspcon2.1 = 1;		/*restart condition*/
	while ( thing == 0)  {}
	thing = 0;	
	sspbuf = 0xD1;		/*slave address READ*/
	while ( thing == 0)  {}
	thing = 0;
	sspcon2.3 = 1;
	while (thing == 0)	{}
	thing = 0;
	sspcon2.4 = 1;		
	sspcon2.5 = 0;
	seconds = sspbuf;  
	/*send ack. bit, store seconds*/
	while (thing == 0)	{}
	thing = 0;
	sspcon2.3 = 1;
	while (thing == 0)	{}
	thing = 0;
	sspcon2.4 = 1;		
	sspcon2.5 = 0;	
	minutes = sspbuf;
	 /*send ack. bit, store minutes*/
	while (thing == 0)	{}
	thing = 0;
	sspcon2.3 = 1;
	while (thing == 0)	{}
	thing = 0;  
	sspcon2.4 = 1;		
	sspcon2.5 = 0;	
	hours = sspbuf;	
	/*send ack. bit, store hours*/
	while (thing == 0)	{}
	thing = 0;
	sspcon2.3 = 1;
	while (thing == 0)   {}
	thing = 0;
	sspcon2.4 = 1;		
	sspcon2.5 = 0;
	day = sspbuf;	/*send ack. bit, continue but don't store anything here*/
	while (thing == 0)	{}
	thing = 0;
	sspcon2.3 = 1;
	while (thing == 0)	{}
	thing = 0;
	sspcon2.4 = 1;		
	sspcon2.5 = 0;
	date = sspbuf;
	/*send ack. bit, store date*/
	while (thing == 0)	{}
	thing = 0;	
	sspcon2.3 = 1;
	while (thing == 0)  {}
	thing = 0; /*send NOT ack. bit, store month*/
	sspcon2.4 = 1;		
	sspcon2.5 = 1;
	month = sspbuf;
	while (thing == 0)	{}
	thing = 0;	
	sspcon2.2 = 1;
	while( thing == 0) {}
	thing = 0;
	
/*PROCESSING SEQUENCE*/

/*SECONDS*/

		short tenseconds;
		short oneseconds;

		tenseconds = seconds / 16;
		oneseconds = seconds % 16;
		seconds = (tenseconds * 10) + oneseconds;

/*MINUTES*/

		short tenminutes;
		short oneminutes;
	
		tenminutes = minutes / 16;
		oneminutes = minutes % 16;
		minutes = (tenminutes * 10) + oneminutes;


/*HOURS*/

		short tenhours;
		short onehours;
		
		tenhours = (hours / 16);
		onehours = hours % 16;
		hours = (tenhours * 10) + onehours;
		hours_temp = hours;
		
		if(!check_twentyfour()){
			hours_temp = hours_temp%12;
			if(!hours_temp){
			hours_temp = 12;
			}
		}

/*DATE*/

		short onedate;
		short tendate;
		
		tendate = date / 16;
		onedate = date % 16;
		date = (tendate * 10) + onedate;

/*MONTH*/

		short tenmonth;
		short onemonth;
		
		tenmonth = (month / 16) % 2;
		onemonth = month % 16;
		month = (tenmonth * 10) + onemonth;
		
/*PRINT TO LCD*/

if(write == false){
	LCD_int(month);
	LCD_printf("/");
	LCD_int(date);
	if (hours < 10){
		LCD_printf("  0");
		LCD_int(hours_temp);
		}
	else{
		LCD_printf("  ");
		LCD_int(hours_temp);
		}
	LCD_printf(":");
	if (minutes < 10){
		LCD_printf("0");
		LCD_int(minutes);
		}
	else{
		LCD_int(minutes);
		}
	//LCD_printf(":");
	//if (seconds < 10){
	//	LCD_printf("0");
	//	LCD_int(seconds);
	//	}
	//else{
	//	LCD_int(seconds);
	//	}
	//
	}
	if (write == true){
		address_low = mem_write_byte(address_low, address_high, minutes);
		address_low = mem_write_byte(address_low, address_high, hours);
		address_low = mem_write_byte(address_low, address_high, date);
		address_low = mem_write_byte(address_low, address_high, month);
		}
	return address_low;
}

unsigned short i2c_data(bool car_on, unsigned short over, unsigned short fault, unsigned short address_low, unsigned short address_high){
	
	volatile bit L1@LATD.0;
	volatile bit L2@LATD.1;
	volatile bit L3@LATD.2;
	volatile bit R1@LATD.3;
	volatile bit R2@LATC.0;
	volatile bit R3@LATC.1;
	
	unsigned short L1_ad = 0x01;	//Camera--approx. 7.5 A (Only for monitoring purposes--1)
	unsigned short L2_ad = 0x05;	//GPS Transciever--15 A (best guess--could not tear up car enough to know for sure--3)
	unsigned short L3_ad = 0x09;	//Motorola 2-way radio--15 A (2)
	unsigned short R1_ad = 0x0D;	//Interceptor Main Power / siren (4)
	unsigned short R2_ad = 0x11;	//floods on lightbar, computer, printer (4)
	unsigned short R3_ad = 0x15;	//Power ports (4)
	unsigned short alt_ad = 0x19;	//alternator current
	unsigned short bata_ad = 0x1D;	//battery current
	unsigned short batv_ad = 0x21;	//battery voltage

	unsigned short L1_cur = 0;	//GPS Transciever
	unsigned short L2_cur = 0;	//?
	unsigned short L3_cur = 0;	//Motorola radio
	unsigned short R1_cur = 0;	//Interceptor Main Power
	unsigned short R2_cur = 0;	//floods on lightbar, computer, printer
	unsigned short R3_cur = 0;	//Power ports 
	
	unsigned short L1_cur_high = 0;
	unsigned short L1_cur_low = 0;
	unsigned short L2_cur_high = 0;
	unsigned short L2_cur_low = 0;
	unsigned short L3_cur_high = 0;
	unsigned short L3_cur_low = 0;
	unsigned short R1_cur_high = 0;
	unsigned short R1_cur_low = 0;
	unsigned short R2_cur_high = 0;
	unsigned short R2_cur_low = 0;
	unsigned short R3_cur_high = 0;
	unsigned short R3_cur_low = 0;
	unsigned short batv_high = 0;
	unsigned short batv_low = 0;
	
	unsigned short alt_cur = 0;
	unsigned short bata_cur = 0;
	unsigned short bata_vol = 0;
	unsigned short soc;
	unsigned short status_register = 0xFF;
	
	L1_cur = check_ad(L1_ad);
	L2_cur = check_ad(L2_ad);
	L3_cur = check_ad(L3_ad);
	R1_cur = check_ad(R1_ad);
	R2_cur = check_ad(R2_ad);
	R3_cur = check_ad(R3_ad);
	alt_cur = check_ad(alt_ad);
	bata_cur = check_ad(bata_ad);
	bata_vol = check_ad(batv_ad);
	
	L1_cur_high = L1_cur / 256;
	L1_cur_low = L1_cur % 256;
	
	L2_cur_high = L2_cur / 256;
	L2_cur_low = L2_cur % 256;
	
	L3_cur_high = L3_cur / 256;
	L3_cur_low = L3_cur % 256;
	
	R1_cur_high = R1_cur / 256;
	R1_cur_low = R1_cur % 256;
	
	R2_cur_high = R2_cur / 256;
	R2_cur_low = R2_cur % 256;
	
	R3_cur_high = R3_cur / 256;
	R3_cur_low = R3_cur % 256;
	
	batv_high = bata_vol / 256;
	batv_low = bata_vol % 256; 
	
	if(L1 == 0){
		status_register = status_register - 128;
		}
	if(L2 == 0){
		status_register = status_register - 64;
		}
	if(L3 == 0){
		status_register = status_register - 32;
		}
	if(R1 == 0){
		status_register = status_register - 16;
		}
	if(R2 == 0){
		status_register = status_register - 8;
		}
	if(R3 == 0){
		status_register = status_register - 4;
		}
	if(car_on == false){
		status_register = status_register - 2;
		}
	if(over == 0){
		status_register = status_register - 1;
		}
	
	address_low = mem_write_byte(address_low, address_high, L1_cur_high);
	address_low = mem_write_byte(address_low, address_high, L1_cur_low);
	address_low = mem_write_byte(address_low, address_high, L2_cur_high);
	address_low = mem_write_byte(address_low, address_high, L2_cur_low);
	address_low = mem_write_byte(address_low, address_high, L3_cur_high);
	address_low = mem_write_byte(address_low, address_high, L3_cur_low);
	address_low = mem_write_byte(address_low, address_high, R1_cur_high);
	address_low = mem_write_byte(address_low, address_high, R1_cur_low);
	address_low = mem_write_byte(address_low, address_high, R2_cur_high);
	address_low = mem_write_byte(address_low, address_high, R2_cur_low);
	address_low = mem_write_byte(address_low, address_high, R3_cur_high);
	address_low = mem_write_byte(address_low, address_high, R3_cur_low);
	address_low = mem_write_byte(address_low, address_high, alt_cur);
	address_low = mem_write_byte(address_low, address_high, bata_cur);
	address_low = mem_write_byte(address_low, address_high, batv_high);
	address_low = mem_write_byte(address_low, address_high, batv_low);
	address_low = mem_write_byte(address_low, address_high, status_register);
	address_low = mem_write_byte(address_low, address_high, fault);
	
	return address_low;
}

void time_init_clk(unsigned short minutes, unsigned short hours, unsigned short date, unsigned short month ){

	volatile bit thing@PIR1.3;
	short addr = 0x01;
	
	sspcon2.0 = 1;
	while ( thing == 0) {}
	thing = 0;
	sspbuf = 0xD0; /* 11010000, code for time chip WRITE*/
	while( thing == 0) {}
	thing = 0;
	sspbuf = addr;	/*start at the minute register*/
	while( thing == 0) {}
	thing = 0;
	sspbuf = minutes; /*initialize minutes*/
	while( thing == 0) {}
	thing = 0;
	sspbuf = hours; /*initialize hours*/
	while( thing == 0) {}
	thing = 0;
	sspbuf = 1;
	while( thing == 0) {}
	thing = 0;
	sspbuf = date; /*initialize date*/
	while( thing == 0) {}
	thing = 0;
	sspbuf = month; /*initialize month*/
	while( thing == 0) {}
	thing = 0;
	sspcon2.2 = 1;
	while( thing == 0) {}
	thing = 0;
}

	/*end of time initialization sequence--now to chip settings
	These commands enable the timer to output an interrupt on
	the */

void time_init_set(void){

	volatile bit thing@PIR1.3;
	short stat = 1;
	while(stat == 1){
		sspcon2.0 = 1;
		while (thing == 0) {}
		thing = 0;
		sspbuf = 0xD0;
		while (thing == 0) {}
		if(sspcon2.6 == 0){
			stat = 0;
			}
		thing = 0;
		}
	sspbuf = 0x0B;
	while (thing == 0) {}
	thing = 0;
	sspbuf = 0x00;
	while (thing == 0) {}
	thing = 0;
	sspbuf = 0x00;
	while (thing == 0) {}
	thing = 0;
	sspbuf = 0x00;
	while (thing == 0) {}
	thing = 0;
	sspbuf = 0x00;
	while (thing == 0) {}
	thing = 0;
	sspcon2.2 = 1;
	while (thing == 0) {}
	thing = 0;
}
	
/*time write hasn't been updated to reflect our work with voltage write, but we
don't see an issue integrating this with what we've learned.*/


/*this functions writes the measured voltage to memory (at the moment, it's the
only functional memory write command, although what's done here is easily 
extendable to whatever other types of data we'd like to write to the memory 
chip.  The front end includes a waiting scheme that allows the uc to finish 
whatever it is doing before performing a write*/

unsigned short mem_write_byte(unsigned short address_low, unsigned short address_high, short value){ 

	/*MEMORY SLAVE ADDRESS: 000*/
	
	short stat = 1;
	
	volatile bit thing@PIR1.3;
	short spacing = 2;        //correspoding to # of bytes we're writing, obvy.
	
	while(stat == 1){		//Wait for slave to finish what it's doing
		i2c_start();
		sspbuf = 0xA0;		//Write to memory address command
		while (thing == 0) {}	
		if (sspcon2.6 == 1){	//If NOT ACK (still writing)
			thing = 0;
			sspcon2.2 = 1;		//Send STOP
			while(thing == 0){}
			thing = 0;
			}
		else{					//If ACK (sspcon2.6 = 0)
			stat = 0;			//Break while loop
			}
		}
	thing = 0;
	sspbuf = address_high;         //Send top half of address
	while( thing == 0) {}
	thing = 0;
	sspbuf = address_low;          //Send bottom half of address
	while( thing == 0) {}
	thing = 0;
	sspbuf = value;         //send top half of voltage measurement
	while( thing == 0) {}
	thing = 0;
	sspcon2.2 = 1;            //stop
	while (thing == 0) {}
	thing = 0;
	
	address_low = address_low + 1;
	return address_low;
}

//////////////////Mode Selection and Switching////////////////////////////////////////

/*This function will use the battery voltage and current readings off of the car
battery (and possibly current being drawn by various devices, if we get that
far?), and combined with the on/off status of the car, return a numeric
code to the main program which determines which devices will be allowed off or
on.*/
unsigned short control_routine(unsigned short over, bool car_on, unsigned short wait_time){

	volatile bit L1@LATD.0;
	volatile bit L2@LATD.1;
	volatile bit L3@LATD.2;
	volatile bit R1@LATD.3;
	volatile bit R2@LATC.0;
	volatile bit R3@LATC.1;
	
	unsigned short switch_status = 0;
	unsigned short fault = 0;
	
	unsigned short wt_1 = 30;
	unsigned short wt_2 = 15;
	unsigned short wt_3 = 60;
	
	unsigned short soc;
	unsigned short switchoff = 0;		//switch off device
	unsigned short switchon = 1;		//switch on device
	bool fault_alt = false;
	bool fault_soc = false;
	bool fault_both = false;
	

	unsigned short L1_ad = 0x01;	//Camera--approx. 7.5 A (Only for monitoring purposes--1)
	unsigned short L2_ad = 0x05;	//GPS Transciever--15 A (best guess--could not tear up car enough to know for sure--3)
	unsigned short L3_ad = 0x09;	//Motorola 2-way radio--15 A (2)
	unsigned short R1_ad = 0x0D;	//Interceptor Main Power / siren (4)
	unsigned short R2_ad = 0x11;	//floods on lightbar, computer, printer (4)
	unsigned short R3_ad = 0x15;	//Power ports (4)
	unsigned short alt_ad = 0x19;	//alternator current
	unsigned short bata_ad = 0x1D;	//battery current
	unsigned short batv_ad = 0x21;	//battery voltage

	
	unsigned short L1_cur = 0;	//GPS Transciever
	unsigned short L2_cur = 0;	//?
	unsigned short L3_cur = 0;	//Motorola radio
	unsigned short R1_cur = 0;	//Interceptor Main Power
	unsigned short R2_cur = 0;	//floods on lightbar, computer, printer
	unsigned short R3_cur = 0;	//Power ports 
	unsigned short alt_cur = 0;
	unsigned short bata_cur = 0;
	
	L1_cur = check_ad(L1_ad);
	L2_cur = check_ad(L2_ad);
	L3_cur = check_ad(L3_ad);
	R1_cur = check_ad(R1_ad);
	R2_cur = check_ad(R2_ad);
	R3_cur = check_ad(R3_ad);
	alt_cur = check_ad(alt_ad);
	bata_cur = check_ad(bata_ad);
	
	if((car_on == true) && (bata_cur > alt_cur)){ //If car is on but battery isn't charging, report an error
		fault_alt = true;
	}
	
	if(car_on == false){
		soc = soc_check();
		if(soc < 20){
			fault_soc = true;
			}
		}
		
	if((fault_soc == true) && (fault_alt == true)){
		fault_both = true;
		}
		
	//OVERRIDE AND "ALL SYSTEMS GO" SETUP--stays in place ten minutes after car is shut off
	if((over == 1) || (((car_on == true) || ((car_on == false) && (wait_time == wt_2))) && (fault_alt == false && fault_soc == false))){
		L1 = switchon;	//override makes the system a passive element; all equipment is allowed to operate
		L2 = switchon;	//we 
		L3 = switchon;
		R1 = switchon;
		R2 = switchon;
		R3 = switchon;
		switch_status = 0xFC; //all on, status code 00
	}
	
	//CAR ON WITH BATTERY PROBLEM--also stays in place ten minutes after car is shut off
	if((car_on == true || ((car_on == false) && (wait_time == wt_2))) && ((fault_alt == true) || (fault_soc == true))){
		L1 = switchon;	//camera is always on
		L3 = switchon;	//camera and radio stay on to maintain communication link
		switch_status = 0xFD;  //status code 01; on/off will be adjusted later on.
		
		if(R3 != 0 && R3_cur > 3){	//deactivate cigarette lighter-type power ports (if current over .3 A is being drawn)
			R3 = switchoff;
			switch_status = switch_status - 4;
		}
		
		if(L2 != 0 && L2_cur > 3){	//turn off GPS transciever and radio modem (if current over .3 A is being drawn)
			L2 = switchoff;
			switch_status = switch_status - 64;
		}
		
		if((R2 == 0)  && (R1 != 0) && (R1_cur > 3)){	//turn off siren IAOI floods have already been shut down
			R1 = switchoff;
			switch_status = switch_status - 16;
		}
		
		if((R2 != 0) && (R2_cur > 60)){	//Turns off flood lights (will remain on if only computer/printer are on)
			R2 = switchoff;
			switch_status = switch_status - 8;
		}
	}

	
	if((car_on == false) && (wait_time == wt_3)){
		L1 = switchon;	//camera is always on
		switch_status = 0xFE;  //status code 01; on/off will be adjusted later on.
		
		if(L3 != 0){	//if the car's been off a while, shut the radio off.  Small current, so we don't include 
			L3 = 0;		//criteria for current being drawn.
			switch_status = switch_status - 32;
		}
		
		if((R3 != 0) && (R3_cur > 3)){	//deactivate cigarette lighter-type power ports (if current is being drawn)
			R3 = 0;
			switch_status = switch_status - 4;
		}
		
		if(L2 != 0 && L2_cur > 3){	//turn off GPS transciever and radio modem (if it's drawing current)
			L2 = 0;
			switch_status = switch_status - 64;
		}
		
		if((R1 != 0) && (R1_cur > 3)){	//turn off siren
			R1 = 0;
			switch_status = switch_status - 16;
		}
		
		if((R2 != 0) && (R2_cur > 3)){	//Turns off flood lights, computer, printer
			R2 = 0;
			switch_status = switch_status - 8;
		}
	}
	if (fault_alt == true){
		fault = 2;
		}
	if (fault_soc == true){
		fault = 1;
		}
	if (fault_both == true){
		fault = 3;
		}
	return fault;
}




/***** Timer0 - Time Out Functions ******
These functions are for setting the timer for 
timing out of the menu functions
*****************************************/ 


// Initialize Timer0
void time_out_init(void){
	t0con = 0xC7;
	//intcon.5 = 1;
	// set clock to zero
	tmr0l = 0;
	tmr0h = 0;
}

// Reset Timer0
unsigned short reset_timer(void){
	// set clock to zero
	tmr0l = 0;
	tmr0h = 0;
	return 0;
}


// This resets the timer and increments the COUNT 
unsigned short timer_count(unsigned short count, unsigned short time_lim){
	
	if (count < time_lim){
	reset_timer();
	count = count + 1;
	}
	
	return count;
	
}

// This deterimines when to stop a menu function
short timer_bust(unsigned short count, unsigned short time_lim){

	short bust = 0;

	if(count >= time_lim){
	bust = 1;
	}
	
	return bust;
	 
}	


// Switch Loads For Demo
void load_switch_demo(void){

volatile bit L1@LATD.0; // L1 Camera--approx. 7.5 A (Only for monitoring purposes--1)
volatile bit L2@LATD.1; // L2 GPS Transciever--15 A (best guess--could not tear up car enough to know for sure--3)
volatile bit L3@LATD.2; // L3 Motorola 2-way radio--15 A (2)
volatile bit R1@LATD.3; // R1 Interceptor Main Power / siren (4)
volatile bit R2@LATC.0; // R2 floods on lightbar, computer, printer (4)
volatile bit R3@LATC.1; // R3 Power ports (4)
	
volatile bit up@PORTB.3; // PIN B3
volatile bit down@PORTB.4; // PIN B4
volatile bit set@PORTB.5; // PIN B5


unsigned short count = 0;
unsigned short time_lim = 300;	
short num = 0;
short bounce = 1;


// Display Prompt	
	LCD_clear();
	LCD_setpos(0,0);
	LCD_printf("Press U to Scroll");
	LCD_setpos(1,0);
	LCD_printf("Press D to Switch");



	short bust = 0;			
	while(!bust){


// If UP is pressed, scroll through options 				
			if(up){
				count = reset_timer();
				num = (num + 1)%6;
				LCD_clear();
				LCD_setpos(0,0);
				LCD_printf("Switch");
				LCD_setpos(1,0);
				
				if(num == 0){
				LCD_printf("Camera");
				}
				
				if(num == 1){
				LCD_printf("GPS");
				}
				
				if(num == 2){
				LCD_printf("2-Way Radio");
				}
				
				if(num == 3){
				LCD_printf("Interceptor/Siren");
				}

				if(num == 4){
				LCD_printf("Floods,Comp,Printer");
				}
				
				if(num == 5){
				LCD_printf("Power Port");
				}
								
				delay_s(bounce);
				
			}
			
// If DOWN is pressed SWITCH current option ON/OFF			
			if(down){		
				count = reset_timer();
				
				LCD_clear();
				LCD_setpos(0,0);
				LCD_printf("SWITCHED");
				LCD_setpos(1,0);
				
				if(num == 0){
				LCD_printf("Camera");
				L1 = !L1;
				}
				
				if(num == 1){
				LCD_printf("GPS");
				L2 = !L2;
				}
				
				if(num == 2){
				LCD_printf("2-Way Radio");
				L3 = !L3;
				}
				
				if(num == 3){
				LCD_printf("Interceptor/Siren");
				R1 = !R1;
				}

				if(num == 4){
				LCD_printf("Floods,Comp,Printer");
				R2 = !R2;
				}
				
				if(num == 5){
				LCD_printf("Power Port");
				R3 = !R3;
				}
							
				
				delay_s(2);
			}
		
	
			// check timing out
			if(check_timeoutflag()){
				count = timer_count(count,time_lim);
				bust = timer_bust(count,time_lim);
				write_timeoutflag(0);
			}
	
			if(set){
				count = reset_timer();
				bust = 1;
				LCD_clear();
			}
	
	}
	
	while(set){}
	
	
}



/***************************************************
This converts binary numbers less than 100 to BCD
***************************************************/
unsigned short bin_to_bcd(unsigned short binary){

return (binary/10*16 + binary%10);

} 	 


/**********************************************************************************
This function activates and appropriately orients the menu interrupts
(high-to-low, low-to-high). This is unnecssary in software given that the hardware 
will be fixed, but ensures that if the hardware switch is reveresed, it will 
still function. 
**********************************************************************************/	
void menu_init(void)
{
	volatile bit lcd_menu@PORTB.1; // (pin B1)

	if(lcd_menu){
		intcon2.5 = 0;   //triggers on high-low transition
	}
	
	
	else{
		intcon2.5 = 1;   //triggers on high-low transition
	}
	
		intcon3.3 = 1;    //enable interrupt
		
	trisb.1 = 1; // make port b bit 2 input
	trisb.3 = 1; // make port b bit 3 input
	trisb.4 = 1; // make port b bit 4 input
	trisb.5 = 1; // make port b bit 5 input

}




/*******************************************************
 This function is an LCD menu interface
*******************************************************/
void LCD_main_menu(bool read_all, unsigned short ad_high, unsigned short ad_low, unsigned short ad_high_end, unsigned short ad_low_end)
{

	// diable watch dog timer
	wdtcon.0 = 0;
	
	//enable timer0 interrupt
	time_out_init();
	write_timeoutflag(0);
	intcon.5 = 1;
	
	volatile bit time_out@INTCON.2;
	volatile bit up@PORTB.3; // (pin B3)
	volatile bit down@PORTB.4; // (pin B4)
	volatile bit set@PORTB.5; // (pin B5)
	

	short option = 0;
	// Set option max to number of options
	short option_max = 6;
	
	//These values are for the time out function
	unsigned short count;
	short exit = 0;
	unsigned short time_lim = 250;
	
	
	// Display prompt
	LCD_clear();
	LCD_printf("Menu:");
	LCD_setpos(1,0);
	LCD_printf("Press Up or Down");
	
	
	// Reset Timer and Timeout Flag
	count = reset_timer();
	write_timeoutflag(0);
	
	
	
	while(!exit){
	// if DOWN is pressed, scroll through options descending priority
			if(down){
			count = reset_timer();
			option = (option + 1)%option_max;
			disp_menu_option(option);
			delay_s(1);
			}
			
	// if UP is pressed, scroll through options in ascending priority		
			if(up){
			count = reset_timer();
			option = (option - 1)%option_max;
				if(option < 0){
				option = option_max-1;
				}
			disp_menu_option(option);
			delay_s(1);
			}
			
	// check for timing out		
			if(check_timeoutflag()){
				count = timer_count(count,time_lim);
				exit = timer_bust(count,time_lim);
				write_timeoutflag(0);
			}
	
	// if SET is pressed, run the option		
			if(set){
				exit = run_option(option, read_all, ad_high, ad_low, ad_high_end, ad_low_end);
				if(!exit){
					disp_menu_option(option);
					count = reset_timer();
				}
			}

	}
	
	

		// enable watch dog timer
		wdtcon.0 = 1;
		
		//disable timer0 interrupt
		t0con.7 = 0;
		intcon.5 = 0;
	
}
	
/*********************************************************	
This displays the menu option based on the option number	
*********************************************************/
void disp_menu_option(short option)
{


	volatile bit up@PORTB.3; // (pin B3)
	volatile bit down@PORTB.4; // (pin B4)
	volatile bit set@PORTB.5; // (pin B5)
	
	LCD_clear();
	
	// Override
	if (option == 1){
	LCD_printf("Menu:");
	LCD_setpos(1,0);
	LCD_printf("Set Soft Override");
	}
	
	// A/D Measurements
	else if (option == 2){
	LCD_printf("Menu:");
	LCD_setpos(1,0);
	LCD_printf("Measurements");
	}
	
	// Set Clock Option
	else if (option == 3){
	LCD_printf("Menu:");
	LCD_setpos(1,0);
	LCD_printf("Set Clock");
	}
	
	// Terminal 
	else if (option == 4){
	LCD_printf("Menu:");
	LCD_setpos(1,0);
	LCD_printf("Terminal");	
	}


	// Load Switch Demo
	else if (option == 5){
	LCD_printf("Menu:");
	LCD_setpos(1,0);
	LCD_printf("Load Switch Demo");	
	}


	// Exit Menu
	else if (option == 0){
	LCD_printf("Menu:");
	LCD_setpos(1,0);
	LCD_printf("Exit Menu");
	}
		
}

/*******************************************
This runs the option given in the LCD Menu
********************************************/
short run_option(short option, bool read_all, unsigned short ad_high, unsigned short ad_low, unsigned short ad_high_end, unsigned short ad_low_end)
{

volatile bit up@PORTB.3; // (pin B3)
volatile bit down@PORTB.4; // (pin B4)
volatile bit set@PORTB.5; // (pin B5)	

short exit = 0;

	// prevent from resetting too quickly
	while(set){}
 
 
	// Software Override
	if (option == 1){
		set_override();
	}

	// A/D Measurements
	else if (option == 2){
		load_check();
	}

	// Set Clock
	else if (option == 3){
		set_time();
	}
	
	// Terminal 
	else if (option == 4){
		LCD_clear();
		LCD_setpos(0,0);
		LCD_printf("In Terminal Mode");
		term_menu(read_all, ad_high, ad_low, ad_high_end, ad_low_end);
		LCD_clear();
		LCD_setpos(0,0);
		LCD_printf("Leaving terminal");
		delay_s(1);
	//dumps memory contents into terminal
	}
	
	
	else if (option == 5){
		load_switch_demo();
	}
	
	// Exit Menu
	else{
	// option == 0 
	exit = 1;
	// exit to main program
	}
	
	return exit;
	
}

/*This function is an interface to set the date and time */	
void set_time(void)
{
	
	volatile bit up@PORTB.3; // (pin B3)
	volatile bit down@PORTB.4; // (pin B4)
	volatile bit set@PORTB.5; // (pin B5)
	
	// initialize values
	unsigned short minute = 0;
	unsigned short hour = 0;
	unsigned short date = 1;
	unsigned short month = 1;
	short bounce = 1;
	short mod_date;

	bool success = false;

	unsigned short count = 0;
	short bust = 0;
	// define time limit for timing out
	unsigned short time_lim = 200;
	
	// set month 
	LCD_clear();
	LCD_printf("Set Month: ");
	LCD_int(month);
	
	bust = 0;
	while(!bust){
	
			// if UP, increment month
			while(up){
			
				count = reset_timer();
				month = (month + 1)%12;
			
				if(!month){
					month = 12;
				}
				
				LCD_clear();
				LCD_printf("Set Month: ");
				LCD_int(month);
				delay_s(bounce);
			
			}
		
			// if DOWN, decrement month
			while(down){
				
				count = reset_timer();
				month = (month - 1)%12;
				
				if(!month){
					month = 12;
				}
			
				LCD_clear();
				LCD_printf("Set Month: ");
				LCD_int(month);
				delay_s(bounce);
			
			}
		
			//  check time out
			if(check_timeoutflag()){
				count = timer_count(count,time_lim);
				bust = timer_bust(count,time_lim);
				write_timeoutflag(0);
				success = false;
			}		
			
			// set month
			if(set){
			count = reset_timer();
			success = true;
			bust = 1;
			}					
		
	}
	
		// Display Set Month	
		if(success){
			LCD_clear();
			LCD_printf("Month set to: ");
			LCD_int(month);
			count = reset_timer();
			delay_s(2);
		
	
			// determine MAX number of days in month
			if (month == 1 || month == 3 || month == 5 || month == 7 || month == 8 || month == 10 || month == 12){
				mod_date = 31;
			} 
	
			else if (month == 4 || month == 6 || month == 9 || month == 11){
				mod_date = 30;
			} 
			else {
				mod_date = 29;
			}
		
		}
		


if(success){
	success = false;
	// set date 
	LCD_clear();
	LCD_printf("Set Date: ");
	LCD_int(date);
	
	bust = 0;
	count = reset_timer();
	while(!bust){	

			// if UP, increment date
			while(up){
				count = reset_timer();
				date = (date + 1)%mod_date;
				if(!date){
					date = mod_date;
				}
			LCD_clear();
			LCD_printf("Set Date: ");
			LCD_int(date);
			delay_s(bounce);

			}
			

			// if DOWN, decrement date
			while(down){
				count = reset_timer();			
				date = (date - 1)%mod_date;
				if(!date){
					date = mod_date;
				}
			LCD_clear();
			LCD_printf("Set Date: ");
			LCD_int(date);
			delay_s(bounce);

			}
		
			// check time out
			if(check_timeoutflag()){
				count = timer_count(count,time_lim);
				bust = timer_bust(count,time_lim);
				write_timeoutflag(0);
				success = false;
			}
		
			// set date
			if(set){
				count = reset_timer();
				success = true;
				bust = 1;
			}		


	}
			
			// Display date
			if(success){
				LCD_clear();
				LCD_printf("Date set to: ");
				LCD_int(date);
				count = reset_timer();
				delay_s(2);
			}


}


if(success){
	success = false;
	// set hour
	LCD_clear();
	LCD_printf("Set Hour: ");
	LCD_int(hour);
	bust = 0;
	count = reset_timer();
	while(!bust){	

			// if UP, increment hour
			while(up){
			count = reset_timer();
			hour = (hour + 1)%24;

			LCD_clear();
			LCD_printf("Set Hour: ");
			LCD_int(hour);
			delay_s(bounce);

			}
			
			// if DOWN, decrement hour
			while(down){
			count = reset_timer();
			if(!hour){
			hour = 24;
			}
			hour = (hour - 1)%24;	
			LCD_clear();
			LCD_printf("Set Hour: ");
			LCD_int(hour);
			delay_s(bounce);
			}
		
			// check timing out
			if(check_timeoutflag()){
				count = timer_count(count,time_lim);
				bust = timer_bust(count,time_lim);
				write_timeoutflag(0);
				success = false;
			}
			
			// set hour			
			if(set){
				count = reset_timer();
				success = true;
				bust = 1;
			}
			
			// display hour
			if(success){
				LCD_clear();
				LCD_printf("Hour set to: ");
				LCD_int(hour);
				count = reset_timer();
				delay_s(2);
				}
	
	}
}


if(success){
	success = false;
	// set minute
	LCD_clear();
	LCD_printf("Set Minutes: ");
	LCD_int(minute);

	bust = 0;
	count = reset_timer();
	while(!bust){

		// if UP, increment minute
		while(up){
			count = reset_timer();
			minute = (minute + 1)%60;
			LCD_clear();
			LCD_printf("Set Minutes: ");
			LCD_int(minute);
			delay_s(bounce);

			}
			
		// if DOWN, decrement minute	
		while(down){
			count = reset_timer();
			if(!minute){
			minute = 60;
			}
			minute = (minute - 1)%60;
			LCD_clear();
			LCD_printf("Set Minutes: ");
			LCD_int(minute);
			delay_s(bounce);
			
			}

		// check timing out
		if(check_timeoutflag()){
				count = timer_count(count,time_lim);
				bust = timer_bust(count,time_lim);
				write_timeoutflag(0);
				success = false;
			}
		
		// set minute	
		if(set){
				success = true;
				bust = 1;
			}
		
		// Display Minute
		if(success){
			LCD_clear();
			LCD_printf("Minute set to: ");
			LCD_int(minute);
			delay_s(2);
			}
	}	

 }



if(success){
	success = false;
// 12/24 hour display
short tf;
tf = check_twentyfour();



// Display Prompt		
LCD_clear();
LCD_setpos(0,0);
LCD_printf("Press U/D to toggle:");
LCD_setpos(1,0);
	if(tf){
		LCD_printf("24 Hour Display");
	}
	else{
		LCD_printf("12 Hour Display");
	}
	delay_s(bounce);
bust = 0;



while(!bust){

			// If UP or DOWN, toggle between 12/24 Hour Display
			if(up ||down){
			count = reset_timer();
			tf = !tf;
			LCD_setpos(1,0);
				if(tf){
				LCD_printf("24 Hour Display");
				}
				else{
				LCD_printf("12 Hour Display");
				}
			
			delay_s(bounce);
			}
		
			// check timing out
			if(check_timeoutflag()){
				count = timer_count(count,time_lim);
				bust = timer_bust(count,time_lim);
				write_timeoutflag(0);
				success = false;
			}
		
		
			// set clock display settings
			if(set){
				count = reset_timer();
				success = true;
				bust = 1;
			}
}		
	
		
		// Display clock display setting
		if(success){
			write_twentyfour(tf);
			LCD_setpos(0,0);
			LCD_printf("Display set to:      ");
			count = reset_timer();
			delay_s(bounce);
		}	
		
}



if(success){

// covnvert values from binary to BCD
minute = bin_to_bcd(minute);
hour = bin_to_bcd(hour);
date = bin_to_bcd(date);
month = bin_to_bcd(month);

// Display Time Set 
LCD_clear();
time_init_clk( minute, hour, date, month );// initiate clock time
LCD_setpos(0,0);
LCD_printf("Time set to:");
LCD_setpos(1,0);
i2c_time(false,0,0);
delay_s(5);
time_init_set();
}

else{
// Display TIMEOUT 
LCD_setpos(0,0);
LCD_clear();
LCD_printf("Operation Timed Out");
delay_s(5);
}

}


/*********************************
Set software override in LCD Menu
**********************************/
void set_override(void){

volatile bit up@PORTB.3; // (pin B3)
volatile bit down@PORTB.4; // (pin B4)
volatile bit set@PORTB.5; // (pin B5)

unsigned short count;
short bust = 0;
unsigned short time_lim = 250;
bool success = false;
short bounce = 1;
unsigned short ovr;
ovr = check_override();


// Display Prompt		
LCD_clear();
LCD_setpos(0,0);
LCD_printf("Press U/D to toggle:");
LCD_setpos(1,0);
	if(ovr){
		LCD_printf("Override  ON");
	}
	else{
		LCD_printf("Override OFF");
	}
	delay_s(bounce);

bust = 0;
count = reset_timer();
write_timeoutflag(0);

while(!bust){
			// if UP or DOWN, toggle override
			if(up || down){
			count = reset_timer();
			ovr = !ovr;
			LCD_setpos(1,0);
				if(ovr){
				LCD_printf("Override  ON");
				}
				else{
				LCD_printf("Override OFF");
				}
			
			delay_s(bounce);
			}
			
			// check time out		
			if(check_timeoutflag()){
				count = timer_count(count,time_lim);
				bust = timer_bust(count,time_lim);
				write_timeoutflag(0);
				success = false;
			}
			
			// set override
			if(set){
				bust = 1;
				success = true;
			}

		
	}
		
		if(success){
			// Display Override setting
			write_override(ovr);
			LCD_setpos(0,0);
			LCD_printf("Set mode to:        ");
			delay_s(5);
		}
		
		else{
			// Display Time Out 
			LCD_clear();
			write_override(ovr);
			LCD_setpos(0,0);
			LCD_printf("Operation Timed Out!");
			LCD_setpos(1,0);
			LCD_printf("Override Unchanged");
			delay_s(5);
		}

}


void load_check(void){

volatile bit up@PORTB.3; // (pin B3)
volatile bit down@PORTB.4; // (pin B4)
volatile bit set@PORTB.5; // (pin B5)
	
unsigned short L1_ad = 0x01;	//Camera--approx. 7.5 A (Only for monitoring purposes--1)
unsigned short L2_ad = 0x05;	//GPS Transciever--15 A (best guess--could not tear up car enough to know for sure--3)
unsigned short L3_ad = 0x09;	//Motorola 2-way radio--15 A (2)
unsigned short R1_ad = 0x0D;	//Interceptor Main Power / siren (4)
unsigned short R2_ad = 0x11;	//floods on lightbar, computer, printer (4)
unsigned short R3_ad = 0x15;	//Power ports (4)
unsigned short alt_ad = 0x19;	//alternator current
unsigned short bata_ad = 0x1D;	//battery current
unsigned short batv_ad = 0x21;	//battery voltage
int adnum = 0;
short bounce = 1;

unsigned short count;
short bust = 0;
unsigned short time_lim = 250;

bust = 0;
count = reset_timer();
write_timeoutflag(0);


while(set){}

while(!bust){

		// if UP, ascend through options
		if(up){
			count = reset_timer();
			adnum = (adnum + 1)%9;
			delay_s(bounce);
			}
		// if DOWN, descend through options	
		else if(down){
			count = reset_timer();
			adnum = (adnum - 1)%9;
				if(adnum < 0){
					adnum = 8;
				}
			delay_s(bounce);
		}
		
		// if idle, display measurement for option
		else if(!up ||!down){
			LCD_clear();
			
			if(adnum == 0){
				LCD_setpos(0,0);
				LCD_printf("Camera:");
				LCD_setpos(1,0);
				print_ad_LCD(L1_ad);
			}
			
			else if(adnum == 1){
				LCD_setpos(0,0);
				LCD_printf("GPS Transciever:");
				LCD_setpos(1,0);
				print_ad_LCD(L2_ad);
			}
			
			else if(adnum == 2){
				LCD_setpos(0,0);
				LCD_printf("2 Way Radio:");
				LCD_setpos(1,0);
				print_ad_LCD(L3_ad);
			}
			
			else if(adnum == 3){
				LCD_setpos(0,0);
				LCD_printf("Interceptor Main");
				LCD_setpos(1,0);
				LCD_printf("Power/Siren: ");
				print_ad_LCD(R1_ad);
			}
			
			else if(adnum == 4){
				LCD_setpos(0,0);
				LCD_printf("Lightbar Floods,");
				LCD_setpos(1,0);
				LCD_printf("Comp,Printer: ");
				print_ad_LCD(R2_ad);
			}
			
			else if(adnum == 5){
				LCD_setpos(0,0);
				LCD_printf("Power Ports:");
				LCD_setpos(1,0);
				print_ad_LCD(R3_ad);
			}

			else if(adnum == 6){
				LCD_setpos(0,0);
				LCD_printf("Alternator Current:");
				LCD_setpos(1,0);
				print_ad_LCD(alt_ad);
			}		
			
			else if(adnum == 7){
				LCD_setpos(0,0);
				LCD_printf("Battery Current:");
				LCD_setpos(1,0);
				print_ad_LCD(bata_ad);
			}
			
			else if(adnum == 8){
				LCD_setpos(0,0);
				LCD_printf("Battery Voltage");
				LCD_setpos(1,0);
				print_ad_LCD(batv_ad);
			}
		
			delay_s(1);

			
		}
		
		
		// check time out
		if(check_timeoutflag()){
			count = timer_count(count,time_lim);
			bust = timer_bust(count,time_lim);
			write_timeoutflag(0);
		}
			
		
		// escape if SET
		if(set){
			bust = 1;
		}		
	

 }



 while(set){}

 
}